/*
<:copyright-gpl 
 Copyright 2007 Broadcom Corp. All Rights Reserved. 
 
 This program is free software; you can distribute it and/or modify it 
 under the terms of the GNU General Public License (Version 2) as 
 published by the Free Software Foundation. 
 
 This program is distributed in the hope it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
 for more details. 
 
 You should have received a copy of the GNU General Public License along 
 with this program; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place - Suite 330, Boston MA 02111-1307, USA. 
:>
*/
/**************************************************************************
 * File Name  : bcmxtmrtimpl.h
 *
 * Description: This file contains constant definitions and structure
 *              definitions for the BCM6358/BCM6348/BCM6338 ATM/PTM
 *              network device driver.
 ***************************************************************************/

#if !defined(_BCMXTMRTIMPL_H)
#define _BCMXTMRTIMPL_H


#define MAX_DEV_CTXS                16
#define MAX_RFC2684_HDR_SIZE        10
#define ENET_MTU_SIZE               1500
#define ENET_HDR_SIZE               14
#define ENET_CRC_SIZE               4
#define MAX_MTU_SIZE                (ENET_MTU_SIZE + MAX_RFC2684_HDR_SIZE + \
                                     ENET_HDR_SIZE + ENET_CRC_SIZE)
#define RXBUF_HEAD_RESERVE          32
#define RXBUF_NEEDED_SIZE           SKB_DATA_ALIGN(RXBUF_HEAD_RESERVE + \
                                        MAX_MTU_SIZE +                  \
                                        sizeof(struct skb_shared_info))
#define MAX_BUFMEM_BLOCKS           16
#define SKB_ALIGNED_SIZE            ((sizeof(struct sk_buff) + 0x0f) & ~0x0f)
#define RFC1626_MTU                 9180
#define CELL_Q_BUF_SIZE             64
#define DEFAULT_CELL_Q_SIZE         10
#if defined(CONFIG_BCM96358)
#define RXBUF_ALLOC_SIZE            ((RXBUF_NEEDED_SIZE + 63) & ~63)
#define PKT_Q_BUF_SIZE_EXP          (RXBUF_ALLOC_SIZE / 64)
#define CELL_Q_BUF_SIZE_EXP         1
#else
#define RXBUF_ALLOC_SIZE            2048
#define PKT_Q_BUF_SIZE_EXP          11
#define CELL_Q_BUF_SIZE_EXP         6
#endif

#define SAR_RX_INT_ID_BASE          INTERRUPT_ID_ATM_DMA_0
#define SAR_TX_INT_ID_BASE          INTERRUPT_ID_ATM_DMA_4
#define SAR_TIMEOUT                 (HZ/20)
#define INVALID_VCID                0xff

#define CACHE_TO_NONCACHE(x)        ((unsigned)(x)|0xA0000000)
#define NONCACHE_TO_CACHE(x)        ((unsigned)(x)&0x9FFFFFFF)
#define CACHE_TO_PHYS(x)            ((unsigned)(x)&0x1FFFFFFF)
#define NONCACHE_TO_PHYS(x)         ((unsigned)(x)&0x1FFFFFFF)
#define PHYS_TO_CACHE(x)            ((unsigned)(x)|0x80000000)
#define PHYS_TO_NONCACHE(x)         ((unsigned)(x)|0xA0000000)

#define XTMRT_UNINITIALIZED         0
#define XTMRT_INITIALIZED           1
#define XTMRT_RUNNING               2

#define SKB_PROTO_ATM_CELL          0xf000
#define XTM_POLL_DONE               0x80000000

#define REG_IRQ_STATUS_OFS          0
#define REG_IRQ_MASK_OFS            1
#define REG_FREE_PKT_Q_OFS          0
#define REG_RX_PKT_Q_OFS            1
#define REG_FREE_CELL_Q_OFS         2
#define REG_RX_CELL_Q_OFS           3
#define REG_NUM_RX_FREE_QS          4

/* Flag bits for ATM_DMA_BD ulFlags_NextRxBd */
#define BD_FLAG_EOP                 0x80000000 // End Of Packet
#define BD_FLAG_CLP                 0x40000000 // Cell Loss Priority
#define BD_FLAG_CI                  0x20000000 // Congestion Indicator
#define BD_FLAG_NEG                 0x10000000 // Negative length

/* BD bit twiddling. */
#define BD_CT_SHIFT                 27
#define BD_FLAGS_SHIFT              27
#define BD_ADDR_MASK                0x07ffffff
#define BD_SET_ADDR(F,V)            F = ((F & ~BD_ADDR_MASK) | \
                                        VIRT_TO_PHY((UINT32) (V)))
#define BD_SET_CT(F,V)              F = ((F & BD_ADDR_MASK) | \
                                        ((UINT32) (V) << BD_CT_SHIFT))
#define BD_GET_CADDR(T,F) \
    ((F) & BD_ADDR_MASK) ? T (PHYS_TO_CACHE((F) & BD_ADDR_MASK)) : NULL
#define BD_GET_NCADDR(T,F) \
    ((F) & BD_ADDR_MASK) ? T (PHYS_TO_NONCACHE((F) & BD_ADDR_MASK)) : NULL
#define BD_SET_RX_RBL(BD,V) \
    BD->ucRxAalErrors_RblHigh = (BD->ucRxAalErrors_RblHigh & 0x8e) | ((V)>>3); \
    BD->ucRxRblLow  = (BD->ucRxRblLow  & 0x1f) | (((V) & 0x07) << 5)
#define BD_GET_RX_RBL(BD) \
    (((BD->ucRxAalErrors_RblHigh & 0x01) << 3) | ((BD->ucRxRblLow & 0xe0) >> 5)

#define BD_TX_PORT_ID_MASK          0x30
#define BD_TX_PORT_ID_SHIFT         4

#if defined(CONFIG_BCM96358)
#define BD_RX_VCID_MASK             0x0f
#define BD_RX_PORT_ID_MASK          0x30
#define BD_RX_PORT_ID_SHIFT         4
#elif defined(CONFIG_BCM96348)
#define BD_RX_VCID_MASK             0x0f
#define BD_RX_PORT_ID_MASK          0x10
#define BD_RX_PORT_ID_SHIFT         4
#elif defined(CONFIG_BCM96338)
#define BD_RX_VCID_MASK             0x03
#define BD_RX_PORT_ID_MASK          0x04
#define BD_RX_PORT_ID_SHIFT         2
#endif

#define BD_RX_AAL_ERROR_MASK        (0xf0 & ~RXAAL5AAL0_LENGTH_ERROR)

#define CT_TRANSPARENT              0x01
#define CT_OAM_F5_SEGMENT           0x04
#define CT_OAM_F5_END_TO_END        0x05
#define CT_AAL5                     0x07

/* ATM Error Indicators. */
#define RXATM_PORT_NOT_ENABLED      0x80
#define RXATM_HEC_ERROR             0x40
#define RXATM_PTI_ERROR             0x20
#define RXATM_RECEIVED_IDLE_CELL    0x10
#define RXATM_INVALID_VPI_VCI       0x08
#define RXATM_NOT_USED              0x04
#define RXATM_OAM_RM_CRC_ERROR      0x02
#define RXATM_GFC_ERROR             0x01

/* AAL5 and AAL0 Error Indicators. */
#define RXAAL5AAL0_CRC_ERROR        0x80
#define RXAAL5AAL0_SHORT_PKT_ERROR  0x40
#define RXAAL5AAL0_LENGTH_ERROR     0x20
#define RXAAL5AAL0_BIG_PKT_ERROR    0x10

/* Definitions for ulIrqStatus and ulIrqMask field. */
#define INTR_TX_QUEUE                       0x00000001
#define INTR_RCQ_ALMOST_FULL                0x00000002
#define INTR_FCQ_ALMOST_EMPTY               0x00000004
#define INTR_RPQ_ALMOST_FULL                0x00000008
#define INTR_FPQ_ALMOST_EMPTY               0x00000010
#define INTR_MIB_COUNTER_HALF_FULL          0x00000020
#define INTR_RCQ_WD_TIMER                   0x00000040
#define INTR_RPQ_WD_TIMER                   0x00000080
#define INTR_RCQ_IMMED_RSP                  0x00000100
#define INTR_RPQ_IMMED_RSP                  0x00000200
#define INTR_ERROR_RX_RTR_DROPPED_CELL      0x00000400
#define INTR_ERROR_VCAM_MULT_MATCH          0x00000800
#define INTR_TX_QUEUE_ENABLE_ALL            0x00ff0000

#define INTR_MASK (INTR_RCQ_ALMOST_FULL | INTR_RPQ_ALMOST_FULL | \
    INTR_RCQ_WD_TIMER | INTR_RPQ_WD_TIMER | INTR_RCQ_IMMED_RSP | \
    INTR_RPQ_IMMED_RSP | INTR_ERROR_VCAM_MULT_MATCH | INTR_TX_QUEUE_ENABLE_ALL)


/* ATM DMA Buffer Descriptor */
typedef struct AtmDmaBd
{
    UINT32 ulCt_BufPtr;
    UINT32 ulFlags_NextRxBd;
    UINT8 ucUui8;
    UINT8 ucRxPortId_Vcid;
    UINT16 usLength;
    union
    {
        UINT8 ucTxPortId_Gfc;
        UINT8 ucRxAtmErrors;
    } u1;
    UINT8 ucRxAalErrors_RblHigh;
    union
    {
        UINT8 ucRxRblLow;
        UINT8 ucFreeRbl;
    } u2;
    UINT8 ucReserved;
} ATM_DMA_BD, *PATM_DMA_BD;

/* Information about a DMA transmit channel. A device instance may use more
 * than one transmit DMA channel. A DMA channel corresponds to a transmit queue.
 */
typedef struct TxQInfo
{
    UINT32 ulPort;
    UINT32 ulPtmPriority;
    UINT32 ulSubPriority;
    UINT32 ulQueueSize;
    UINT32 ulDmaIndex;
    UINT32 ulUseCount;

    UINT8 *pMemBuf;
    volatile PATM_DMA_BD pQBase;
    volatile UINT32 *pulQHtl;
    volatile UINT8 *pucHeadIdx;
    volatile UINT8 *pucTailIdx;
    UINT32 ulHeadIdx;
    UINT32 ulLength;
    UINT32 ulCurrBdIdx;
    UINT8 ucNumBds;
    struct sk_buff **ppSkbs;
    UINT32 ulFreeBds;
    UINT32 ulHead;
} TXQINFO, *PTXQINFO;

/* The definition of the driver control structure */
typedef struct bcmxtmrt_dev_context
{
    /* Linux structures. */
    struct net_device *pDev;        
    struct net_device_stats DevStats;
    IOCTL_MIB_INFO MibInfo;
    struct ppp_channel Chan;

    /* ATM/PTM fields. */
    XTM_ADDR Addr;
    UINT32 ulLinkState;
    UINT32 ulOpenState;
    UINT32 ulAdminStatus;
    UINT32 ulHdrType;
    UINT32 ulEncapType; /* IPoA, PPPoA, or EoA[bridge,MER,PPPoE] */
    UINT32 ulFlags;

    /* Transmit fields. */
    UINT8 ucTxVcid;
    UINT32 ulTxQueuesSize;
    PTXQINFO pTxQueues[MAX_TRANSMIT_QUEUES];
    PTXQINFO pTxPriorities[MAX_PHY_PORTS][MAX_SUB_PRIORITIES];
} BCMXTMRT_DEV_CONTEXT, *PBCMXTMRT_DEV_CONTEXT;

typedef struct RxFreeQueues
{
    UINT32 ulQAddr;
    UINT32 ulQHead;
    UINT32 ulQTail;
    UINT32 ulQLen;
} RXFREEQUEUES, *PRXFREEQUEUES;

/* Information that is global to all network device instances. */
typedef struct bcmxtmrt_global_info
{
    /* Linux structures. */
    PBCMXTMRT_DEV_CONTEXT pDevCtxs[MAX_DEV_CTXS];
    PBCMXTMRT_DEV_CONTEXT pDevCtxsByVcid[MAX_DEV_CTXS];
    struct tasklet_struct Tasklet;
    struct timer_list Timer;
    struct atm_dev *pAtmDev;

    /* DMA, BD and buffer fields. */
    struct sk_buff *pRxSkbMem;
    struct sk_buff *pFreeRxSkbList;
    UINT8 *pRxBdMem;
    UINT8 *pBufMem[MAX_BUFMEM_BLOCKS];
    UINT8 *pRxCellMem;
    TXQINFO TxQInfos[MAX_TRANSMIT_QUEUES];
    volatile UINT32 *pulIrqStatus;
    volatile UINT32 *pulIrqMask;
    volatile PRXFREEQUEUES pRxFreeQs;
    volatile UINT32 *pulTxQAddrBase;
    volatile UINT32 *pulTxQHtlBase;
    UINT32 *pulRxFreeQAddrs[REG_NUM_RX_FREE_QS];

    /* Callback functions. */
    XTMRT_CALLBACK pfnOamHandler;
    void *pOamContext;
    XTMRT_CALLBACK pfnAsmHandler;
    void *pAsmContext;

    /* Everything else. */
    UINT32 ulDrvState;
} BCMXTMRT_GLOBAL_INFO, *PBCMXTMRT_GLOBAL_INFO;


#endif /* _BCMXTMRTIMPL_H */

